@@ -10,8 +10,8 @@
 """Launch Configuration Dialog"""
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cfgdlg.py 62581 2009-11-10 02:52:54Z CJP $"
-__revision__ = "$Revision: 62581 $"
+__svnid__ = "$Id: cfgdlg.py 67261 2011-03-20 17:29:53Z CJP $"
+__revision__ = "$Revision: 67261 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -25,10 +25,13 @@ import zlib
 import handlers
 
 # Editra Libraries
+import ed_glob
 import eclib
-import util
 import ed_msg
+import ed_basewin
 from profiler import Profile_Get, Profile_Set
+import syntax.syntax as syntax
+import util
 
 #-----------------------------------------------------------------------------#
 # Globals
@@ -43,6 +46,7 @@ ID_EXECUTABLES = wx.NewId()
 # Misc Panel
 ID_AUTOCLEAR = wx.NewId()
 ID_ERROR_BEEP = wx.NewId()
+ID_WRAP_OUTPUT = wx.NewId()
 
 # Color Buttons
 ID_DEF_BACK = wx.NewId()
@@ -59,45 +63,11 @@ COLOR_MAP = { ID_DEF_BACK : 'defaultb', ID_DEF_FORE : 'defaultf',
               ID_INFO_BACK : 'infob',   ID_INFO_FORE : 'infof',
               ID_WARN_BACK : 'warnb',   ID_WARN_FORE : 'warnf'}
 
-# Message Types
-EDMSG_LAUNCH_CFG_EXIT = ed_msg.EDMSG_ALL + ('launch', 'cfg', 'exit')
-
 _ = wx.GetTranslation
-#-----------------------------------------------------------------------------#
-
-def GetMinusData():
-    return zlib.decompress(
-"x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x06$\
-\xc3Jc\x9e\x03)\x96b'\xcf\x10\x0e \xa8\xe1H\xe9\x00\xf2\x9d<]\x1cC4&&\xa7\
-\xa4$\xa5)\xb0\x1aL\\RU\x90\x95\xe0\xf8,\xc6\xaa\xf0\xcf\xffr\x13\xd69\x87\
-\xb8x\xaaVM\xea\x890\xf512N\x9e\xb1v\xf5\xe9\x05\xdc\xc2;jf:\x96\xdf\xd2\x14\
-a\x96pO\xda\xc0\xc4\xa0\xf4\x8a\xab\xcau\xe2|\x1d\xa0i\x0c\x9e\xae~.\xeb\x9c\
-\x12\x9a\x00Ij($" )
-
-def GetMinusBitmap():
-    stream = cStringIO.StringIO(GetMinusData())
-    return wx.BitmapFromImage(wx.ImageFromStream(stream))
-
-#----------------------------------------------------------------------
-def GetPlusData():
-    return zlib.decompress(
-"x\xda\xeb\x0c\xf0s\xe7\xe5\x92\xe2b``\xe0\xf5\xf4p\t\x02\xd2< \xcc\xc1\x06$\
-\xc3Jc\x9e\x03)\x96b'\xcf\x10\x0e \xa8\xe1H\xe9\x00\xf2{<]\x1cC4&&'Hp\x1c\
-\xd8\xb9\xcf\xe6U\xfd\xefi\xbb\xffo\xf44J\x14L\xae\xde\x97+yx\xd3\xe9\xfc\
-\x8d\xb3\xda|\x99\x99g\x1b07\x1b\xd8k\x87\xf1\xea\x18\x1c{\xaa\xec\xfe\xaf>%\
-!\xf9A\xda\xef\x03\x06\xf67{\x1f\x1e\xf8\xf9\x98g\xf9\xb9\xf9\xbf\xfe\xbf~\
-\xad\xcf\x96'h\xca\xe6\xcck\xe8&2\xb7\x8e\x87\xe7\xbfdAB\xfb\xbf\xe0\x88\xbf\
-\xcc\xcc\x7f.\xcbH\xfc{\xfd(\xa0\xe5*\xff\xfd\xff\x06\x06\x1f\xfe\xffh\xbaj\
-\xf2f^ZB\xc2\x83\xe4\xc3\xef2o13<r\xd5y\xc0\xb9\xc2\xfa\x0e\xd0]\x0c\x9e\xae\
-~.\xeb\x9c\x12\x9a\x00\xcf9S\xc6" )
-
-def GetPlusBitmap():
-    stream = cStringIO.StringIO(GetPlusData())
-    return wx.BitmapFromImage(wx.ImageFromStream(stream))
 
 #----------------------------------------------------------------------
 
-class ConfigDialog(wx.Frame):
+class ConfigDialog(ed_basewin.EdBaseFrame):
     """Configuration dialog for configuring what executables are available
     for a filetype and what the preferred one is.
 
@@ -108,22 +78,14 @@ class ConfigDialog(wx.Frame):
         @keyword: The filetype to set
 
         """
-        wx.Frame.__init__(self, parent, title=_("Launch Configuration"))
+        super(ConfigDialog, self).__init__(parent, title=_("Launch Configuration"))
 
         # Layout
-        util.SetWindowIcon(self)
         self.__DoLayout()
 
-        # Event Handlers
-        self.Bind(wx.EVT_CLOSE, self.OnClose)
-
-        # Register with app
-        wx.GetApp().RegisterWindow(repr(self), self)
-
     def __DoLayout(self):
         """Layout the dialog"""
         sizer = wx.BoxSizer(wx.VERTICAL)
-
         hsizer = wx.BoxSizer(wx.HORIZONTAL)
         vsizer = wx.BoxSizer(wx.VERTICAL)
         panel = wx.Panel(self)
@@ -137,17 +99,12 @@ class ConfigDialog(wx.Frame):
         self.SetInitialSize()
         self.SetMinSize((420, 345))
 
-    def OnClose(self, evt):
-        """Unregister the window when its closed"""
-        wx.GetApp().UnRegisterWindow(repr(self))
-        evt.Skip()
-
 #-----------------------------------------------------------------------------#
 
 class ConfigNotebook(wx.Notebook):
     """Notebook for holding config pages"""
     def __init__(self, parent):
-        wx.Notebook.__init__(self, parent)
+        super(ConfigNotebook, self).__init__(parent)
 
         # Make sure config has been initialized
         prefs = Profile_Get(LAUNCH_PREFS, default=None)
@@ -157,6 +114,7 @@ class ConfigNotebook(wx.Notebook):
             Profile_Set(LAUNCH_PREFS,
                         dict(autoclear=False,
                              errorbeep=False,
+                             wrapoutput=False,
                              defaultf=buff.GetDefaultForeground().Get(),
                              defaultb=buff.GetDefaultBackground().Get(),
                              errorf=buff.GetErrorForeground().Get(),
@@ -169,17 +127,20 @@ class ConfigNotebook(wx.Notebook):
 
         # Setup
         self.AddPage(ConfigPanel(self), _("General"))
+        self.AddPage(OutputPanel(self), _("Output"))
         self.AddPage(MiscPanel(self), _("Misc"))
 
-    def __del__(self):
-        ed_msg.PostMessage(EDMSG_LAUNCH_CFG_EXIT)
-
 #-----------------------------------------------------------------------------#
 
 class ConfigPanel(wx.Panel):
     """Configuration panel that holds the controls for configuration"""
     def __init__(self, parent):
-        wx.Panel.__init__(self, parent)
+        super(ConfigPanel, self).__init__(parent)
+
+        # Attributes
+        self.addbtn = None
+        self.delbtn = None
+        self.infotxt = None
 
         # Layout
         self.__DoLayout()
@@ -195,15 +156,17 @@ class ConfigPanel(wx.Panel):
 
         lsizer = wx.BoxSizer(wx.HORIZONTAL)
         ftype = wx.GetApp().GetCurrentBuffer().GetLangId()
-        ftype = handlers.GetHandlerById(ftype).GetName()
-        htypes = GetHandlerTypes()
+        ftype = handlers.GetHandlerById(ftype)
+        hname = ftype.GetName()
+        util.Log("[Launch][info] ConfigPanel: %s" % hname)
+        htypes = sorted(syntax.SyntaxNames())
         lang_ch = wx.Choice(self, ID_LANGUAGE, choices=htypes)
         if ftype != handlers.DEFAULT_HANDLER:
-            lang_ch.SetStringSelection(ftype)
+            lang_ch.SetStringSelection(hname)
         else:
             lang_ch.SetStringSelection(htypes[0])
 
-        lsizer.AddMany([(wx.StaticText(self, label=_("File Type") + ":"), 0,
+        lsizer.AddMany([(wx.StaticText(self, label=_("File Type") + u":"), 0,
                          wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                         (lang_ch, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)])
 
@@ -223,7 +186,7 @@ class ConfigPanel(wx.Panel):
         else:
             pass
 
-        dsizer.AddMany([(wx.StaticText(self, label=_("Default") + ":"), 0,
+        dsizer.AddMany([(wx.StaticText(self, label=_("Default") + u":"), 0,
                          wx.ALIGN_CENTER_VERTICAL), ((5, 5), 0),
                         (def_ch, 1, wx.EXPAND|wx.ALIGN_CENTER_VERTICAL)])
 
@@ -232,16 +195,18 @@ class ConfigPanel(wx.Panel):
                                   style=wx.LC_EDIT_LABELS|\
                                         wx.BORDER|wx.LC_REPORT|\
                                         wx.LC_SINGLE_SEL)
-#        exelist.SetToolTipString(_("Click on an item to edit"))
-#        exelist.InsertColumn(0, _("Alias"))
-#        exelist.InsertColumn(1, _("Executable Commands"))
         self.SetListItems(chandler.GetCommands())
-        addbtn = wx.BitmapButton(self, wx.ID_ADD, GetPlusBitmap())
-        addbtn.SetToolTipString(_("Add a new executable"))
-        delbtn = wx.BitmapButton(self, wx.ID_REMOVE, GetMinusBitmap())
-        delbtn.SetToolTipString(_("Remove selection from list"))
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_ADD), wx.ART_MENU)
+        self.addbtn = wx.BitmapButton(self, wx.ID_ADD, bmp)
+        self.addbtn.SetToolTipString(_("Add a new executable"))
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_REMOVE), wx.ART_MENU)
+        self.delbtn = wx.BitmapButton(self, wx.ID_REMOVE, bmp)
+        self.delbtn.SetToolTipString(_("Remove selection from list"))
         btnsz = wx.BoxSizer(wx.HORIZONTAL)
-        btnsz.AddMany([(addbtn, 0), ((2, 2), 0), (delbtn, 0)])
+        self.infotxt = wx.StaticText(self, label=_("Transient XML Handler"))
+        self.infotxt.Hide()
+        btnsz.AddMany([(self.addbtn, 0), ((2, 2), 0), (self.delbtn, 0),
+                       ((5,2),0), (self.infotxt, 0, wx.ALIGN_CENTER_VERTICAL)])
 
         # Box Sizer Layout
         boxsz.AddMany([((5, 5), 0), (dsizer, 0, wx.ALIGN_CENTER|wx.EXPAND),
@@ -259,7 +224,8 @@ class ConfigPanel(wx.Panel):
         hsizer = wx.BoxSizer(wx.HORIZONTAL)
         hsizer.AddMany([((8, 8), 0), (msizer, 1, wx.EXPAND), ((8, 8), 0)])
         self.SetSizer(hsizer)
-        self.SetAutoLayout(True)
+
+        self.UpdateForHandler()
 
     def __DoUpdateHandler(self, handler):
         exes = self.GetListItems()
@@ -267,6 +233,7 @@ class ConfigPanel(wx.Panel):
         def_ch = self.FindWindowById(wx.ID_DEFAULT)
         def_ch.SetItems(handler.GetAliases())
         def_ch.SetStringSelection(handler.GetDefault())
+        handler.StoreState()
 
     def GetCurrentHandler(self):
         """Get the currently selected file type handler
@@ -284,7 +251,7 @@ class ConfigPanel(wx.Panel):
         item_id = -1
         exes = list()
         elist = self.FindWindowById(ID_EXECUTABLES)
-        for item in xrange(elist.GetItemCount()):
+        for item in range(elist.GetItemCount()):
             item_id = elist.GetNextItem(item_id)
             if item_id == -1:
                 break
@@ -301,7 +268,7 @@ class ConfigPanel(wx.Panel):
         e_id = evt.GetId()
         elist = self.FindWindowById(ID_EXECUTABLES)
         if e_id == wx.ID_ADD:
-            elist.Append([_("**Alias**"), _("**New Value**")])
+            elist.Append([_("**Alias**"), _("**New Commandline**")])
         elif e_id == wx.ID_REMOVE:
             item = -1
             items = []
@@ -316,7 +283,6 @@ class ConfigPanel(wx.Panel):
                 elist.DeleteItem(item)
 
             wx.CallAfter(self.__DoUpdateHandler, self.GetCurrentHandler())
-
         else:
             evt.Skip()
 
@@ -326,16 +292,11 @@ class ConfigPanel(wx.Panel):
         e_obj = evt.GetEventObject()
         e_val = e_obj.GetStringSelection()
         if e_id == ID_LANGUAGE:
-            handler = handlers.GetHandlerByName(e_val)
-            elist = self.FindWindowById(ID_EXECUTABLES)
-            elist.DeleteAllItems()
-            def_ch = self.FindWindowById(wx.ID_DEFAULT)
-            def_ch.SetItems(handler.GetAliases())
-            def_ch.SetStringSelection(handler.GetDefault())
-            self.SetListItems(handler.GetCommands())
+            self.UpdateForHandler()
         elif e_id == wx.ID_DEFAULT:
             handler = self.GetCurrentHandler()
             handler.SetDefault((e_val, handler.GetCommand(e_val)))
+            handler.StoreState()
         else:
             evt.Skip()
 
@@ -361,13 +322,14 @@ class ConfigPanel(wx.Panel):
             else:
                 # Add a new item
                 # This should not happen
-                if col == 0:
-                    exes.append((nval, nval))
-                else:
-                    exes.append((nval, nval))
+                exes.append((nval, nval))
 
             # Store the new values
             handler.SetCommands(exes)
+            if len(exes) == 1:
+                # Make sure default is set
+                handler.SetDefault(exes[0])
+            handler.StoreState()
             def_ch = self.FindWindowById(wx.ID_DEFAULT)
             def_ch.SetItems(handler.GetAliases())
             def_ch.SetStringSelection(handler.GetDefault())
@@ -381,14 +343,29 @@ class ConfigPanel(wx.Panel):
         for exe in items:
             elist.Append(exe)
 
+    def UpdateForHandler(self):
+        handler = self.GetCurrentHandler()
+        elist = self.FindWindowById(ID_EXECUTABLES)
+        elist.DeleteAllItems()
+        def_ch = self.FindWindowById(wx.ID_DEFAULT)
+        def_ch.SetItems(handler.GetAliases())
+        def_ch.SetStringSelection(handler.GetDefault())
+        self.SetListItems(handler.GetCommands())
+        # Enable control states
+        transient = handler.meta.transient
+        def_ch.Enable(not transient)
+        elist.Enable(not transient)
+        self.addbtn.Enable(not transient)
+        self.delbtn.Enable(not transient)
+        if self.infotxt.Show(transient):
+            self.Layout()
+
 #-----------------------------------------------------------------------------#
 
-class MiscPanel(wx.Panel):
-    """Misc settings panel"""
+class OutputPanel(wx.Panel):
+    """Output buffer settings panel"""
     def __init__(self, parent):
-        wx.Panel.__init__(self, parent)
-
-        # Attributes
+        super(OutputPanel, self).__init__(parent)
 
         # Layout
         self.__DoLayout()
@@ -408,11 +385,14 @@ class MiscPanel(wx.Panel):
 
         # Actions Configuration
         clear_cb = wx.CheckBox(self, ID_AUTOCLEAR,
-                               _("Automatically clear buffer between runs"))
+                               _("Automatically clear output buffer between runs"))
         clear_cb.SetValue(cfg.get('autoclear', False))
         error_cb = wx.CheckBox(self, ID_ERROR_BEEP,
                                _("Audible feedback when errors are detected"))
         error_cb.SetValue(cfg.get('errorbeep', False))
+        wrap_cb = wx.CheckBox(self, ID_WRAP_OUTPUT,
+                              _("Wrap lines in output buffer"))
+        wrap_cb.SetValue(cfg.get('wrapoutput', False))
 
         # Colors
         colors = dict()
@@ -465,6 +445,7 @@ class MiscPanel(wx.Panel):
                         (wx.StaticText(self, label=("Actions") + u":"), 0),
                         ((5, 5), 0), (clear_cb, 0),
                         ((5, 5), 0), (error_cb, 0),
+                        ((5, 5), 0), (wrap_cb, 0),
                         ((10, 10), 0), (wx.StaticLine(self), 0, wx.EXPAND),
                         ((10, 10), 0),
                         (boxsz, 1, wx.EXPAND)])
@@ -481,6 +462,8 @@ class MiscPanel(wx.Panel):
             cfg['autoclear'] = e_val
         elif e_id == ID_ERROR_BEEP:
             cfg['errorbeep'] = e_val
+        elif e_id == ID_WRAP_OUTPUT:
+            cfg['wrapoutput'] = e_val
         else:
             evt.Skip()
 
@@ -493,20 +476,62 @@ class MiscPanel(wx.Panel):
         else:
             evt.Skip()
 
+#-----------------------------------------------------------------------------#
+
+class MiscPanel(wx.Panel):
+    """Misc settings panel"""
+    def __init__(self, parent):
+        super(MiscPanel, self).__init__(parent)
+
+        # Attributes
+        self.savecb = None
+        self.saveallcb = None
+
+        # Layout
+        self.__DoLayout()
+
+        # Event Handlers
+        self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
+
+    def __DoLayout(self):
+        """Layout the panel"""
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        cfg = Profile_Get(LAUNCH_PREFS, default=dict())
+
+        # Editor options
+        ebox = wx.StaticBox(self, label=_("Editor Options"))
+        eboxsz = wx.StaticBoxSizer(ebox, wx.VERTICAL)
+        lbl = _("Automatically save current file before running")
+        self.savecb = wx.CheckBox(self, label=lbl)
+        self.savecb.SetValue(cfg.get('autosave', False))
+        lbl = _("Automatically save all open files before running")
+        self.saveallcb = wx.CheckBox(self, label=lbl)
+        self.saveallcb.SetValue(cfg.get('autosaveall', False))
+        eboxsz.Add(self.savecb, 0, wx.ALL, 3)
+        eboxsz.Add(self.saveallcb, 0, wx.ALL, 3)
+        sizer.Add(eboxsz, 0, wx.EXPAND|wx.ALL, 5)
+
+        self.SetSizer(sizer)
+
+    def OnCheck(self, event):
+        """Update the configuration"""
+        e_obj = event.GetEventObject()
+        value = e_obj.GetValue()
+        cfg = Profile_Get(LAUNCH_PREFS, default=dict())
+        if e_obj is self.savecb:
+            cfg['autosave'] = value
+        elif e_obj is self.saveallcb:
+            cfg['autosaveall'] = value
+        else:
+            event.Skip()
+
 #-----------------------------------------------------------------------------#
 ID_BROWSE = wx.NewId()
 
-class CommandListCtrl(listmix.ListCtrlAutoWidthMixin,
-                      listmix.TextEditMixin,
-                      eclib.ListRowHighlighter,
-#                      listmix.CheckListCtrlMixin,
-                      wx.ListCtrl):
+class CommandListCtrl(eclib.EEditListCtrl):
     """Auto-width adjusting list for showing editing the commands"""
     def __init__(self, *args, **kwargs):
-        wx.ListCtrl.__init__(self, *args, **kwargs)
-        listmix.ListCtrlAutoWidthMixin.__init__(self)
-#        listmix.CheckListCtrlMixin.__init__(self)
-        eclib.ListRowHighlighter.__init__(self)
+        super(CommandListCtrl, self).__init__(*args, **kwargs)
 
         # Attributes
         self._menu = None
@@ -520,8 +545,6 @@ class CommandListCtrl(listmix.ListCtrlAutoWidthMixin,
         self.InsertColumn(1, _("Executable Commands"))
 #        self.SetColumnWidth(0, self.GetTextExtent(pcol)[0] + 5)
 
-        listmix.TextEditMixin.__init__(self)
-
         # Event Handlers
         self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.OnContextClick)
         self.Bind(wx.EVT_MENU, self.OnMenu, id=ID_BROWSE)
@@ -595,19 +618,3 @@ class CommandListCtrl(listmix.ListCtrlAutoWidthMixin,
 #        """
 #        parent = self.GetParent()
 #        parent.UpdateCurrentHandler(index)
-
-#-----------------------------------------------------------------------------#
-
-def GetHandlerTypes():
-    """Get the language type handlers for each language that
-    has a handler defined.
-    @return: list of handler names
-
-    """
-    keys = handlers.HANDLERS.keys()
-    keys.remove(0)
-    rlist = list()
-    for key in keys:
-        handle = handlers.HANDLERS[key]
-        rlist.append(handle.GetName().title())
-    return sorted(rlist)
