@@ -18,8 +18,8 @@ element is defined in the file.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cbrowser.py 58863 2009-02-13 00:29:52Z CJP $"
-__revision__ = "$Revision: 58863 $"
+__svnid__ = "$Id: cbrowser.py 66072 2010-11-08 21:04:45Z CJP $"
+__revision__ = "$Revision: 66072 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -33,6 +33,7 @@ from profiler import Profile_Get, Profile_Set
 import ed_msg
 
 # Local Imports
+import cbconfig
 import gentag.taglib as taglib
 from tagload import TagLoader
 import IconFile
@@ -53,7 +54,7 @@ STRINGS = ( _("Class Definitions"), _("Defines"), _("Function Definitions"),
             _("Sections"), _("Style Tags"), _("Subroutines"),
             _("Subroutine Declarations"), _("Task Definitions"), 
             _("Modules"), _("Functions"), _("Public Functions"),
-            _("Public Subroutines") )
+            _("Public Subroutines"), _("Imports"), _("Elements") )
 del STRINGS
 
 #--------------------------------------------------------------------------#
@@ -62,9 +63,10 @@ class CodeBrowserTree(wx.TreeCtrl):
     def __init__(self, parent, id=ID_BROWSER,
                  pos=wx.DefaultPosition, size=wx.DefaultSize,
                  style=wx.TR_DEFAULT_STYLE|wx.TR_HIDE_ROOT):
-        wx.TreeCtrl.__init__(self, parent, id, pos, size, style)
+        super(CodeBrowserTree, self).__init__(parent, id, pos, size, style)
 
         # Attributes
+        self._log = wx.GetApp().GetLog()
         self._mw = parent
         self._menu = None
         self._selected = None
@@ -74,6 +76,14 @@ class CodeBrowserTree(wx.TreeCtrl):
         self.icons = dict()
         self.il = None
 
+        # struct used in buffer-tree sync
+        self._ds_flat = list() # list of tuples - [(line, item_id),(line, item_id)...]
+
+        self._timer = wx.Timer(self)
+        self._sync_timer = wx.Timer(self)
+        self._cpage = None
+        self._force = False
+
         # Setup
         self._SetupImageList()
         viewm = self._mw.GetMenuBar().GetMenuByName("view")
@@ -90,12 +100,21 @@ class CodeBrowserTree(wx.TreeCtrl):
         # Event Handlers
         self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivated)
         self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnContext)
+        self.Bind(wx.EVT_TREE_ITEM_EXPANDED, self.OnExpanding)
         self.Bind(wx.EVT_MENU, self.OnMenu)
+        self.Bind(wx.EVT_TIMER, self.OnStartJob, self._timer)
+        self.Bind(wx.EVT_TIMER, lambda evt: self._SyncTree(), self._sync_timer)
         self.Bind(EVT_JOB_FINISHED, self.OnTagsReady)
+
+        # Editra Message Handlers
         ed_msg.Subscribe(self.OnThemeChange, ed_msg.EDMSG_THEME_CHANGED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_UI_NB_CHANGED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_FILE_OPENED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_FILE_SAVED)
+        ed_msg.Subscribe(self.OnSyncTree, ed_msg.EDMSG_UI_STC_POS_CHANGED)
+        ed_msg.Subscribe(self.OnEditorRestore, ed_msg.EDMSG_UI_STC_RESTORE)
+        ed_msg.Subscribe(self.OnBrowserCfg,
+                         ed_msg.EDMSG_PROFILE_CHANGE + (cbconfig.CB_PROFILE_KEY,))
 
         # Backwards compatibility
         if hasattr(ed_msg, 'EDMSG_UI_STC_LEXER') and \
@@ -108,28 +127,81 @@ class CodeBrowserTree(wx.TreeCtrl):
         ed_msg.Unsubscribe(self.OnUpdateTree)
         ed_msg.Unsubscribe(self.OnThemeChange)
         ed_msg.Unsubscribe(self.OnUpdateFont)
+        ed_msg.Unsubscribe(self.OnSyncTree)
+        ed_msg.Unsubscribe(self.OnEditorRestore)
 
     def _GetCurrentCtrl(self):
         """Get the current buffer"""
         return self._mw.GetNotebook().GetCurrentCtrl()
 
+    def _GetFQN(self, node):
+        """Returns fully qualified name of the tree node in the form
+        ...grandparent.parent.child
+        @param node: node id
+
+        """
+        rval=""
+        if node and node.IsOk():
+            path = [self.GetItemText(node)]
+            parent = self.GetItemParent(node)
+            root = self.GetRootItem()
+            while parent and parent.IsOk() and parent != root:
+                path.append(self.GetItemText(parent))
+                parent = self.GetItemParent(parent)
+            
+            # Join path elements and strip line information as this could be changed
+            path.reverse()
+            rval = '.'.join([p.split(u'[')[0].strip() for p in path])
+            
+        return rval
+    
+    def _ClearTree(self):
+        """Clear the tree and caches"""
+        self._cdoc = None
+        self._ds_flat = list()
+        self.DeleteChildren(self.root)
+
+    def _FindNodeForLine(self, line):
+        """Returns node id of the docstruct element given line belongs to
+        @param line: line number
+        @returns: tree node id
+
+        """
+        # HACK This should probably be done with bisect search
+        rval = None
+        line += 1
+        if self._ds_flat:
+            for citem in self._ds_flat:
+                if citem[0] < line:
+                    cline = citem[0]
+                    rval = citem[1]
+                else:
+                    break
+
+#        self._log("[codebrowser][info] For line %d found item %s" % \
+#                  (line, self._GetFQN(rval)))
+        return rval
+
     def _SetupImageList(self):
         """Setup the image list for the tree"""
         imglst = wx.ImageList(16, 16)
-        if Profile_Get('ICONS', 'Default') != 'Default':
-            globe = wx.ArtProvider.GetBitmap(str(ed_glob.ID_WEB), wx.ART_MENU)
-            self.icons['globals'] = imglst.Add(globe)
-        else:
-            self.icons['globals'] = imglst.Add(IconFile.World.GetBitmap())
-        self.icons['class'] = imglst.Add(IconFile.Bricks.GetBitmap())
+
+        globe = wx.ArtProvider.GetBitmap(str(ed_glob.ID_WEB), wx.ART_MENU)
+        self.icons['globals'] = imglst.Add(globe)
+
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_CLASS_TYPE), wx.ART_MENU)
+        self.icons['class'] = imglst.Add(bmp)
         self.icons['section'] = imglst.Add(IconFile.Brick_Add.GetBitmap())
-        self.icons['function'] = imglst.Add(IconFile.Brick_Go.GetBitmap())
-        self.icons['method'] = self.icons['function']
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_FUNCT_TYPE), wx.ART_MENU)
+        self.icons['function'] = imglst.Add(bmp)
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_METHOD_TYPE), wx.ART_MENU)
+        self.icons['method'] = imglst.Add(bmp)
         self.icons['subroutine'] = self.icons['function']
         self.icons['procedure'] = self.icons['function']
-        self.icons['task'] = imglst.Add(IconFile.Brick_Go2.GetBitmap())
+        self.icons['task'] = imglst.Add(bmp)
         self.icons['function2'] = self.icons['task']
-        self.icons['variable'] = imglst.Add(IconFile.Brick.GetBitmap())
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_VARIABLE_TYPE), wx.ART_MENU)
+        self.icons['variable'] = imglst.Add(bmp)
         self.icons['namespace'] = imglst.Add(IconFile.Brick_Bricks.GetBitmap())
         self.icons['tag_red'] = imglst.Add(IconFile.Tag_Red.GetBitmap())
         self.icons['tag_blue'] = imglst.Add(IconFile.Tag_Blue.GetBitmap())
@@ -145,15 +217,38 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         """
         img = self.icons.get(cobj.type, None)
+
+        # Try and find an appropriate fallback icon
         if img is None:
             if isinstance(cobj, taglib.Function):
                 img = self.icons['function']
+            elif isinstance(cobj, taglib.Class):
+                img = self.icons['class']
             elif isinstance(cobj, taglib.Scope):
                 img = self.icons['section']
             else:
                 img = self.icons['variable']
         return img
 
+    def _SyncTree(self):
+        """Synchronize tree node selection with current position in the text
+        """
+        if not self._ShouldUpdate():
+            return
+        line = self._GetCurrentCtrl().GetCurrentLineNum()
+        self._log("[codebrowser][info] Syncing tree for position %d" % line)
+        scope_item = self._FindNodeForLine(line)
+        if scope_item:
+            selected = self.GetSelection()
+            if selected:
+                if selected != scope_item:
+                    self.Unselect()
+                    self.SelectItem(scope_item)
+            else:
+                self.SelectItem(scope_item)
+                
+            self.EnsureVisible(scope_item)
+
     def _ShouldUpdate(self):
         """Check whether the tree should do an update or not
         @return: bool
@@ -171,7 +266,10 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         """
         if self.nodes.get('classes', None) is None:
-            croot = self.AppendItem(self.GetRootItem(), _("Class Definitions"))
+            desc = self._cdoc.GetElementDescription('class')
+            if desc == 'class':
+                desc = "Class Definitions"
+            croot = self.AppendItem(self.GetRootItem(), _(desc))
             self.SetItemHasChildren(croot)
             self.SetPyData(croot, None)
             self.SetItemImage(croot, self.icons['class'])
@@ -187,6 +285,7 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         """
         item_id = self.AppendItem(node, u"%s [%d]" % (cobj.GetName(), 1 + cobj.GetLine()), img)
+        self._ds_flat.append((cobj.GetLine(), item_id))
         self.SetPyData(item_id, cobj.GetLine())
         # If the item is a scope it may have sub items
         if isinstance(cobj, taglib.Scope):
@@ -224,7 +323,7 @@ class CodeBrowserTree(wx.TreeCtrl):
         """
         # Check if there is a node for this Code object
         if self.nodes.get(obj.type, None) is None:
-            # Look for a description to use as catagory title
+            # Look for a description to use as category title
             if self._cdoc is not None:
                 desc = self._cdoc.GetElementDescription(obj.type).title()
             else:
@@ -239,10 +338,14 @@ class CodeBrowserTree(wx.TreeCtrl):
 
     def DeleteChildren(self, item):
         """Delete the children of a given node"""
-        wx.TreeCtrl.DeleteChildren(self, item)
+        super(CodeBrowserTree, self).DeleteChildren(item)
         for key in self.nodes.keys():
             self.nodes[key] = None
 
+    def GetMainWindow(self):
+        """Get this panels main window"""
+        return self._mw
+
     def GotoElement(self, tree_id):
         """Navigate the cursor to the element identified in the
         code browser tree.
@@ -253,7 +356,10 @@ class CodeBrowserTree(wx.TreeCtrl):
         if line is not None:
             ctrl = self._mw.GetNotebook().GetCurrentCtrl()
             ctrl.GotoLine(line)
-            ctrl.SetFocus()
+
+            # Need CallAfter to make the focus work on Windows
+            # Otherwise the tree control will reclaim the focus
+            wx.CallAfter(ctrl.SetFocus)
 
     def OnActivated(self, evt):
         """Handle when an item is clicked on
@@ -286,17 +392,39 @@ class CodeBrowserTree(wx.TreeCtrl):
         @return: -1, 0, 1
 
         """
-        txt1 = self.GetItemText(item1).lower()
-        txt2 = self.GetItemText(item2).lower()
-#        txt1 = self.GetPyData(item1)
-#        txt2 = self.GetPyData(item2)
-        if txt1 < txt2:
+        sortopt = cbconfig.GetSortOption()
+        if sortopt == cbconfig.CB_ALPHA_SORT:
+            # Sort items by name
+            val1 = self.GetItemText(item1).lower()
+            val2 = self.GetItemText(item2).lower()
+        else:
+            # Sort items by line number
+            val1 = self.GetPyData(item1)
+            val2 = self.GetPyData(item2)
+
+        if val1 < val2:
             return -1
-        elif txt1 == txt2:
+        elif val1 == val2:
             return 0
         else:
             return 1
 
+    def OnBrowserCfg(self, msg):
+        """Profile update message for when CodeBrowser
+        configuration is updated.
+
+        """
+        # Resort all items
+        def SortAllItems(parent):
+            item, cookie = self.GetFirstChild(parent)
+            while item:
+                if self.ItemHasChildren(item):
+                    if self.IsExpanded(item):
+                        self.SortChildren(item)
+                    SortAllItems(item)
+                item, cookie = self.GetNextChild(parent, cookie)
+        SortAllItems(self.root)
+
     def OnMenu(self, evt):
         """Handle the context menu events"""
         if evt.GetId() == ID_GOTO_ELEMENT:
@@ -305,6 +433,16 @@ class CodeBrowserTree(wx.TreeCtrl):
         else:
             evt.Skip()
 
+    @ed_msg.mwcontext
+    def OnEditorRestore(self, msg):
+        """Called when editor size is unmaximized"""
+        self.OnUpdateTree()
+
+    def OnExpanding(self, evt):
+        """Update sorting"""
+        item = evt.GetItem()
+        self.SortChildren(item)
+
     def OnThemeChange(self, msg):
         """Update the images when Editra's theme changes
         @param msg: Message Object
@@ -330,6 +468,9 @@ class CodeBrowserTree(wx.TreeCtrl):
             ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE,
                                (self._mw.GetId(), 0, 0))
 
+            if not self._timer.IsRunning():
+                self._cpage = None
+
     def OnUpdateFont(self, msg):
         """Update the ui font when a message comes saying to do so."""
         font = msg.GetData()
@@ -345,14 +486,77 @@ class CodeBrowserTree(wx.TreeCtrl):
         pane = self._mw.GetFrameManager().GetPane(PANE_NAME)
         evt.Check(pane.IsShown())
 
+    @ed_msg.mwcontext
+    def OnSyncTree(self, msg):
+        """Handler for tree synchronization.
+        Uses a one shot timer to optimize multiple fast caret movement events.
+
+        """
+        if not self._ShouldUpdate():
+            return # If panel is not visible don't update
+        
+        if self._sync_timer.IsRunning():
+            self._sync_timer.Stop()
+        
+        #One shot timer for tree sync
+        self._sync_timer.Start(300, True)
+
+    def OnStartJob(self, evt):
+        """Start the tree update job
+        
+        @param evt: wxTimerEvent
+
+        """
+        if self._cpage is None or not isinstance(self._cpage, wx.Window):
+            self._cpage = None
+            return
+        else:
+            # Check if its still the current page
+            parent = self._cpage.GetParent()
+            if self._cpage != parent.GetCurrentPage():
+                return
+
+        # Get the generator method
+        genfun = TagLoader.GetGenerator(self._cpage.GetLangId())
+        self._cjob += 1 # increment job Id
+
+        # Check if we need to do updates
+        if genfun is not None and (self._force or self._ShouldUpdate()):
+            self._force = False
+
+            # Start progress indicator in pulse mode
+            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
+                               (self._mw.GetId(), True))
+            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE,
+                               (self._mw.GetId(), -1, -1))
+
+            # Create and start the worker thread
+            thread = TagGenThread(self, self._cjob, genfun,
+                                  StringIO.StringIO(self._cpage.GetText()))
+            wx.CallLater(75, thread.start)
+        else:
+            self._ClearTree()
+            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
+                               (self._mw.GetId(), False))
+            return
+
     def OnUpdateTree(self, msg=None, force=False):
         """Update the tree when an action message is sent
         @keyword msg: Message Object
         @keyword force: Force update
 
         """
-        # Don't update when this window is not Active
-        if not force and not self._mw.IsActive():
+        # Check for if update should be skipped
+        if not force:
+            context = None
+            if msg is not None:
+                context = msg.GetContext()
+
+            if context is not None and context != self._mw.GetId():
+                return
+        
+        # Don't update if panel is not visible
+        if not self._ShouldUpdate():
             return
 
         page = self._GetCurrentCtrl()
@@ -360,36 +564,29 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         # If its a blank document just clear out
         if not len(cfname):
-            self._cdoc = None
-            self.DeleteChildren(self.root)
+            self._ClearTree()
             return
 
         # If document job is same as current don't start a new one
-        if not force and self._lastjob == cfname:
-            return
-        else:
+        if force or self._lastjob != cfname:
             self._lastjob = cfname
+            if self._timer.IsRunning():
+                self._timer.Stop()
 
-        # Get the generator method
-        genfun = TagLoader.GetGenerator(page.GetLangId())
-        self._cjob += 1 # increment job Id
+            # Cache the current job information
+            self._cpage = page
+            self._force = force
 
-        # Check if we need to do updates
-        if genfun is not None and (force or self._ShouldUpdate()):
-            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
-                               (self._mw.GetId(), True))
-            # Start progress indicator in pulse mode
-            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_STATE,
-                               (self._mw.GetId(), -1, -1))
-            thread = TagGenThread(self, self._cjob, genfun,
-                                  StringIO.StringIO(page.GetText()))
-            wx.CallLater(75, thread.start)
-        else:
-            self._cdoc = None
-            self.DeleteChildren(self.root)
-            ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
-                               (self._mw.GetId(), False))
-            return
+            # Start the oneshot timer for beginning the tag generator job
+            self._timer.Start(300, True)
+    
+    def OnShowAUIPane(self):
+        """Interface method that the main Editra window will call
+        When its auimanager does a Show when this window is contained
+        within an aui pane. Forces the tree to update.
+
+        """
+        self.OnUpdateTree(force=True)
 
     def OnShowBrowser(self, evt):
         """Show the browser pane
@@ -401,7 +598,7 @@ class CodeBrowserTree(wx.TreeCtrl):
             pane = mgr.GetPane(PANE_NAME)
             pane.Show(not pane.IsShown())
             mgr.Update()
-            self.OnUpdateTree(force=True)
+            self.OnShowAUIPane()
         else:
             evt.Skip()
 
@@ -410,8 +607,8 @@ class CodeBrowserTree(wx.TreeCtrl):
         @param tags: DocStruct object
 
         """
+        self._ClearTree()
         self._cdoc = tags
-        self.DeleteChildren(self.root)
         # Check and add any common types in the document first
 
         # Global Variables
@@ -433,6 +630,9 @@ class CodeBrowserTree(wx.TreeCtrl):
         for node in [ node for node in self.nodes.values()
                       if node is not None and node != self.nodes['globals']]:
             self.Expand(node)
+        
+        self._ds_flat.sort(cmp=lambda x,y: cmp(x[0],y[0]))
+        self._SyncTree()
 
 #--------------------------------------------------------------------------#
 # Tag Generator Thread
